# flake8: noqa
import base64

from codenerix_lib.normalizers import strong_normalizer


def test_strong_normalizer():
    b64test = """U2FsdGFyIGFsIGNvbnRlbmlkbyBwcmluY2lwYWwKICAgICAgICAgICAgICAgIE1lbsO6CgogICAg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"""

    b64expected = """c2FsdGFyIGFsIGNvbnRlbmlkbyBwcmluY2lwYWwgbWVudSBkaWFyaSBvZmljaWFsIGRlIGxhIGdl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"""

    test = base64.b64decode(b64test).decode()
    expected = base64.b64decode(b64expected).decode()
    assert strong_normalizer(test) == expected
    assert strong_normalizer(test, remove_allspaces=True) == expected.replace(
        " ",
        "",
    )
